// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_vx_fcs_fcv_h)
#define octave_vx_fcs_fcv_h 1
#include "octave-config.h"
#include "fCColVector.h"
#include "oct-cmplx.h"
#include "fColVector.h"
  extern OCTAVE_API FloatComplexColumnVector operator + (const FloatComplex&, const FloatColumnVector&);
  extern OCTAVE_API FloatComplexColumnVector operator - (const FloatComplex&, const FloatColumnVector&);
  extern OCTAVE_API FloatComplexColumnVector operator * (const FloatComplex&, const FloatColumnVector&);
  extern OCTAVE_API FloatComplexColumnVector operator / (const FloatComplex&, const FloatColumnVector&);
#endif
