// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_ui64nda_nda_h)
#define octave_mx_ui64nda_nda_h 1
#include "octave-config.h"
#include "uint64NDArray.h"
#include "dNDArray.h"
  extern OCTAVE_API uint64NDArray operator + (const uint64NDArray&, const NDArray&);
  extern OCTAVE_API uint64NDArray operator - (const uint64NDArray&, const NDArray&);
  extern OCTAVE_API uint64NDArray product (const uint64NDArray&, const NDArray&);
  extern OCTAVE_API uint64NDArray quotient (const uint64NDArray&, const NDArray&);
  extern OCTAVE_API boolNDArray mx_el_lt (const uint64NDArray&, const NDArray&);
  extern OCTAVE_API boolNDArray mx_el_le (const uint64NDArray&, const NDArray&);
  extern OCTAVE_API boolNDArray mx_el_ge (const uint64NDArray&, const NDArray&);
  extern OCTAVE_API boolNDArray mx_el_gt (const uint64NDArray&, const NDArray&);
  extern OCTAVE_API boolNDArray mx_el_eq (const uint64NDArray&, const NDArray&);
  extern OCTAVE_API boolNDArray mx_el_ne (const uint64NDArray&, const NDArray&);
  extern OCTAVE_API boolNDArray mx_el_and (const uint64NDArray&, const NDArray&);
  extern OCTAVE_API boolNDArray mx_el_or (const uint64NDArray&, const NDArray&);
  extern OCTAVE_API boolNDArray mx_el_not_and (const uint64NDArray&, const NDArray&);
  extern OCTAVE_API boolNDArray mx_el_not_or (const uint64NDArray&, const NDArray&);
  extern OCTAVE_API boolNDArray mx_el_and_not (const uint64NDArray&, const NDArray&);
  extern OCTAVE_API boolNDArray mx_el_or_not (const uint64NDArray&, const NDArray&);
#endif
