// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_smx_scm_m_h)
#define octave_smx_scm_m_h 1
#include "octave-config.h"
#include "CMatrix.h"
#include "CSparse.h"
#include "dMatrix.h"
#include "mx-cs-m.h"
#include "mx-m-cs.h"
#include "mx-cm-m.h"
#include "mx-m-cm.h"
#include "mx-cs-nda.h"
#include "mx-nda-cs.h"
#include "Sparse-op-defs.h"
  extern OCTAVE_API ComplexMatrix operator + (const SparseComplexMatrix&, const Matrix&);
  extern OCTAVE_API ComplexMatrix operator - (const SparseComplexMatrix&, const Matrix&);
  extern OCTAVE_API SparseComplexMatrix product (const SparseComplexMatrix&, const Matrix&);
  extern OCTAVE_API SparseComplexMatrix quotient (const SparseComplexMatrix&, const Matrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_lt (const SparseComplexMatrix&, const Matrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_le (const SparseComplexMatrix&, const Matrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_ge (const SparseComplexMatrix&, const Matrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_gt (const SparseComplexMatrix&, const Matrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_eq (const SparseComplexMatrix&, const Matrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_ne (const SparseComplexMatrix&, const Matrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_and (const SparseComplexMatrix&, const Matrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_or (const SparseComplexMatrix&, const Matrix&);
#endif
