// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_vx_frv_fcs_h)
#define octave_vx_frv_fcs_h 1
#include "octave-config.h"
#include "fCRowVector.h"
#include "fRowVector.h"
#include "oct-cmplx.h"
  extern OCTAVE_API FloatComplexRowVector operator + (const FloatRowVector&, const FloatComplex&);
  extern OCTAVE_API FloatComplexRowVector operator - (const FloatRowVector&, const FloatComplex&);
  extern OCTAVE_API FloatComplexRowVector operator * (const FloatRowVector&, const FloatComplex&);
  extern OCTAVE_API FloatComplexRowVector operator / (const FloatRowVector&, const FloatComplex&);
#endif
