// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_pm_fm_h)
#define octave_mx_pm_fm_h 1
#include "octave-config.h"
#include "fMatrix.h"
#include "PermMatrix.h"
  extern OCTAVE_API FloatMatrix operator * (const PermMatrix&, const FloatMatrix&);
#endif
