// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_ui16_ui32nda_h)
#define octave_mx_ui16_ui32nda_h 1
#include "octave-config.h"
#include "oct-inttypes.h"
#include "uint32NDArray.h"
  extern OCTAVE_API boolNDArray mx_el_lt (const octave_uint16&, const uint32NDArray&);
  extern OCTAVE_API boolNDArray mx_el_le (const octave_uint16&, const uint32NDArray&);
  extern OCTAVE_API boolNDArray mx_el_ge (const octave_uint16&, const uint32NDArray&);
  extern OCTAVE_API boolNDArray mx_el_gt (const octave_uint16&, const uint32NDArray&);
  extern OCTAVE_API boolNDArray mx_el_eq (const octave_uint16&, const uint32NDArray&);
  extern OCTAVE_API boolNDArray mx_el_ne (const octave_uint16&, const uint32NDArray&);
  extern OCTAVE_API boolNDArray mx_el_and (const octave_uint16&, const uint32NDArray&);
  extern OCTAVE_API boolNDArray mx_el_or (const octave_uint16&, const uint32NDArray&);
  extern OCTAVE_API boolNDArray mx_el_and_not (const octave_uint16&, const uint32NDArray&);
  extern OCTAVE_API boolNDArray mx_el_or_not (const octave_uint16&, const uint32NDArray&);
#endif
