// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_vx_cv_ccv_h)
#define octave_vx_cv_ccv_h 1
#include "octave-config.h"
#include "CColVector.h"
#include "dColVector.h"
  extern OCTAVE_API ComplexColumnVector operator + (const ColumnVector&, const ComplexColumnVector&);
  extern OCTAVE_API ComplexColumnVector operator - (const ColumnVector&, const ComplexColumnVector&);
  extern OCTAVE_API ComplexColumnVector product (const ColumnVector&, const ComplexColumnVector&);
  extern OCTAVE_API ComplexColumnVector quotient (const ColumnVector&, const ComplexColumnVector&);
#endif
