// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_cm_m_h)
#define octave_mx_cm_m_h 1
#include "octave-config.h"
#include "CMatrix.h"
#include "dMatrix.h"
  extern OCTAVE_API ComplexMatrix operator + (const ComplexMatrix&, const Matrix&);
  extern OCTAVE_API ComplexMatrix operator - (const ComplexMatrix&, const Matrix&);
  extern OCTAVE_API ComplexMatrix product (const ComplexMatrix&, const Matrix&);
  extern OCTAVE_API ComplexMatrix quotient (const ComplexMatrix&, const Matrix&);
  extern OCTAVE_API boolMatrix mx_el_lt (const ComplexMatrix&, const Matrix&);
  extern OCTAVE_API boolMatrix mx_el_le (const ComplexMatrix&, const Matrix&);
  extern OCTAVE_API boolMatrix mx_el_ge (const ComplexMatrix&, const Matrix&);
  extern OCTAVE_API boolMatrix mx_el_gt (const ComplexMatrix&, const Matrix&);
  extern OCTAVE_API boolMatrix mx_el_eq (const ComplexMatrix&, const Matrix&);
  extern OCTAVE_API boolMatrix mx_el_ne (const ComplexMatrix&, const Matrix&);
  extern OCTAVE_API boolMatrix mx_el_and (const ComplexMatrix&, const Matrix&);
  extern OCTAVE_API boolMatrix mx_el_or (const ComplexMatrix&, const Matrix&);
#endif
