// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_vx_fs_fcrv_h)
#define octave_vx_fs_fcrv_h 1
#include "octave-config.h"
#include "fCRowVector.h"
  extern OCTAVE_API FloatComplexRowVector operator + (const float&, const FloatComplexRowVector&);
  extern OCTAVE_API FloatComplexRowVector operator - (const float&, const FloatComplexRowVector&);
  extern OCTAVE_API FloatComplexRowVector operator * (const float&, const FloatComplexRowVector&);
  extern OCTAVE_API FloatComplexRowVector operator / (const float&, const FloatComplexRowVector&);
#endif
