// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_vx_ops_h)
#define octave_vx_ops_h 1
#include "octave-config.h"
#include "vx-ccv-cv.h"
#include "vx-ccv-s.h"
#include "vx-crv-rv.h"
#include "vx-crv-s.h"
#include "vx-cs-cv.h"
#include "vx-cs-rv.h"
#include "vx-cv-ccv.h"
#include "vx-cv-cs.h"
#include "vx-rv-crv.h"
#include "vx-rv-cs.h"
#include "vx-s-ccv.h"
#include "vx-s-crv.h"
#include "vx-fccv-fcv.h"
#include "vx-fccv-fs.h"
#include "vx-fcrv-frv.h"
#include "vx-fcrv-fs.h"
#include "vx-fcs-fcv.h"
#include "vx-fcs-frv.h"
#include "vx-fcv-fccv.h"
#include "vx-fcv-fcs.h"
#include "vx-frv-fcrv.h"
#include "vx-frv-fcs.h"
#include "vx-fs-fccv.h"
#include "vx-fs-fcrv.h"
#endif
