// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_vx_fccv_fcv_h)
#define octave_vx_fccv_fcv_h 1
#include "octave-config.h"
#include "fCColVector.h"
#include "fColVector.h"
  extern OCTAVE_API FloatComplexColumnVector operator + (const FloatComplexColumnVector&, const FloatColumnVector&);
  extern OCTAVE_API FloatComplexColumnVector operator - (const FloatComplexColumnVector&, const FloatColumnVector&);
  extern OCTAVE_API FloatComplexColumnVector product (const FloatComplexColumnVector&, const FloatColumnVector&);
  extern OCTAVE_API FloatComplexColumnVector quotient (const FloatComplexColumnVector&, const FloatColumnVector&);
#endif
