// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_smx_ops_h)
#define octave_smx_ops_h 1
#include "octave-config.h"
#include "smx-sm-cs.h"
#include "smx-cs-sm.h"
#include "smx-scm-s.h"
#include "smx-s-scm.h"
#include "smx-scm-sm.h"
#include "smx-sm-scm.h"
#include "smx-m-sm.h"
#include "smx-m-scm.h"
#include "smx-cm-sm.h"
#include "smx-cm-scm.h"
#include "smx-sm-m.h"
#include "smx-scm-m.h"
#include "smx-sm-cm.h"
#include "smx-scm-cm.h"
#include "smx-bm-sbm.h"
#include "smx-sbm-bm.h"
#endif
