// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_vx_fcrv_fs_h)
#define octave_vx_fcrv_fs_h 1
#include "octave-config.h"
#include "fCRowVector.h"
  extern OCTAVE_API FloatComplexRowVector operator + (const FloatComplexRowVector&, const float&);
  extern OCTAVE_API FloatComplexRowVector operator - (const FloatComplexRowVector&, const float&);
  extern OCTAVE_API FloatComplexRowVector operator * (const FloatComplexRowVector&, const float&);
  extern OCTAVE_API FloatComplexRowVector operator / (const FloatComplexRowVector&, const float&);
#endif
