// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_fcdm_fcm_h)
#define octave_mx_fcdm_fcm_h 1
#include "octave-config.h"
#include "fCMatrix.h"
#include "fCDiagMatrix.h"
  extern OCTAVE_API FloatComplexMatrix operator + (const FloatComplexDiagMatrix&, const FloatComplexMatrix&);
  extern OCTAVE_API FloatComplexMatrix operator - (const FloatComplexDiagMatrix&, const FloatComplexMatrix&);
  extern OCTAVE_API FloatComplexMatrix operator * (const FloatComplexDiagMatrix&, const FloatComplexMatrix&);
#endif
