// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_cs_m_h)
#define octave_mx_cs_m_h 1
#include "octave-config.h"
#include "CMatrix.h"
#include "oct-cmplx.h"
#include "dMatrix.h"
  extern OCTAVE_API ComplexMatrix operator + (const Complex&, const Matrix&);
  extern OCTAVE_API ComplexMatrix operator - (const Complex&, const Matrix&);
  extern OCTAVE_API ComplexMatrix operator * (const Complex&, const Matrix&);
  extern OCTAVE_API ComplexMatrix operator / (const Complex&, const Matrix&);
  extern OCTAVE_API boolMatrix mx_el_lt (const Complex&, const Matrix&);
  extern OCTAVE_API boolMatrix mx_el_le (const Complex&, const Matrix&);
  extern OCTAVE_API boolMatrix mx_el_ge (const Complex&, const Matrix&);
  extern OCTAVE_API boolMatrix mx_el_gt (const Complex&, const Matrix&);
  extern OCTAVE_API boolMatrix mx_el_eq (const Complex&, const Matrix&);
  extern OCTAVE_API boolMatrix mx_el_ne (const Complex&, const Matrix&);
  extern OCTAVE_API boolMatrix mx_el_and (const Complex&, const Matrix&);
  extern OCTAVE_API boolMatrix mx_el_or (const Complex&, const Matrix&);
#endif
