// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_smx_sm_cs_h)
#define octave_smx_sm_cs_h 1
#include "octave-config.h"
#include "CMatrix.h"
#include "CSparse.h"
#include "dSparse.h"
#include "oct-cmplx.h"
#include "Sparse-op-defs.h"
  extern OCTAVE_API ComplexMatrix operator + (const SparseMatrix&, const Complex&);
  extern OCTAVE_API ComplexMatrix operator - (const SparseMatrix&, const Complex&);
  extern OCTAVE_API SparseComplexMatrix operator * (const SparseMatrix&, const Complex&);
  extern OCTAVE_API SparseComplexMatrix operator / (const SparseMatrix&, const Complex&);
  extern OCTAVE_API SparseBoolMatrix mx_el_lt (const SparseMatrix&, const Complex&);
  extern OCTAVE_API SparseBoolMatrix mx_el_le (const SparseMatrix&, const Complex&);
  extern OCTAVE_API SparseBoolMatrix mx_el_ge (const SparseMatrix&, const Complex&);
  extern OCTAVE_API SparseBoolMatrix mx_el_gt (const SparseMatrix&, const Complex&);
  extern OCTAVE_API SparseBoolMatrix mx_el_eq (const SparseMatrix&, const Complex&);
  extern OCTAVE_API SparseBoolMatrix mx_el_ne (const SparseMatrix&, const Complex&);
  extern OCTAVE_API SparseBoolMatrix mx_el_and (const SparseMatrix&, const Complex&);
  extern OCTAVE_API SparseBoolMatrix mx_el_or (const SparseMatrix&, const Complex&);
#endif
