// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_fcm_fm_h)
#define octave_mx_fcm_fm_h 1
#include "octave-config.h"
#include "fCMatrix.h"
#include "fMatrix.h"
  extern OCTAVE_API FloatComplexMatrix operator + (const FloatComplexMatrix&, const FloatMatrix&);
  extern OCTAVE_API FloatComplexMatrix operator - (const FloatComplexMatrix&, const FloatMatrix&);
  extern OCTAVE_API FloatComplexMatrix product (const FloatComplexMatrix&, const FloatMatrix&);
  extern OCTAVE_API FloatComplexMatrix quotient (const FloatComplexMatrix&, const FloatMatrix&);
  extern OCTAVE_API boolMatrix mx_el_lt (const FloatComplexMatrix&, const FloatMatrix&);
  extern OCTAVE_API boolMatrix mx_el_le (const FloatComplexMatrix&, const FloatMatrix&);
  extern OCTAVE_API boolMatrix mx_el_ge (const FloatComplexMatrix&, const FloatMatrix&);
  extern OCTAVE_API boolMatrix mx_el_gt (const FloatComplexMatrix&, const FloatMatrix&);
  extern OCTAVE_API boolMatrix mx_el_eq (const FloatComplexMatrix&, const FloatMatrix&);
  extern OCTAVE_API boolMatrix mx_el_ne (const FloatComplexMatrix&, const FloatMatrix&);
  extern OCTAVE_API boolMatrix mx_el_and (const FloatComplexMatrix&, const FloatMatrix&);
  extern OCTAVE_API boolMatrix mx_el_or (const FloatComplexMatrix&, const FloatMatrix&);
#endif
