// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_smx_bm_sbm_h)
#define octave_smx_bm_sbm_h 1
#include "octave-config.h"
#include "boolMatrix.h"
#include "boolSparse.h"
#include "Sparse-op-defs.h"
  extern OCTAVE_API SparseBoolMatrix mx_el_eq (const boolMatrix&, const SparseBoolMatrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_ne (const boolMatrix&, const SparseBoolMatrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_and (const boolMatrix&, const SparseBoolMatrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_or (const boolMatrix&, const SparseBoolMatrix&);
#endif
