// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_cm_pm_h)
#define octave_mx_cm_pm_h 1
#include "octave-config.h"
#include "CMatrix.h"
#include "PermMatrix.h"
  extern OCTAVE_API ComplexMatrix operator * (const ComplexMatrix&, const PermMatrix&);
#endif
