// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_nda_ui64_h)
#define octave_mx_nda_ui64_h 1
#include "octave-config.h"
#include "uint64NDArray.h"
#include "dNDArray.h"
#include "oct-inttypes.h"
  extern OCTAVE_API uint64NDArray operator + (const NDArray&, const octave_uint64&);
  extern OCTAVE_API uint64NDArray operator - (const NDArray&, const octave_uint64&);
  extern OCTAVE_API uint64NDArray operator * (const NDArray&, const octave_uint64&);
  extern OCTAVE_API uint64NDArray operator / (const NDArray&, const octave_uint64&);
  extern OCTAVE_API boolNDArray mx_el_lt (const NDArray&, const octave_uint64&);
  extern OCTAVE_API boolNDArray mx_el_le (const NDArray&, const octave_uint64&);
  extern OCTAVE_API boolNDArray mx_el_ge (const NDArray&, const octave_uint64&);
  extern OCTAVE_API boolNDArray mx_el_gt (const NDArray&, const octave_uint64&);
  extern OCTAVE_API boolNDArray mx_el_eq (const NDArray&, const octave_uint64&);
  extern OCTAVE_API boolNDArray mx_el_ne (const NDArray&, const octave_uint64&);
  extern OCTAVE_API boolNDArray mx_el_and (const NDArray&, const octave_uint64&);
  extern OCTAVE_API boolNDArray mx_el_or (const NDArray&, const octave_uint64&);
  extern OCTAVE_API boolNDArray mx_el_not_and (const NDArray&, const octave_uint64&);
  extern OCTAVE_API boolNDArray mx_el_not_or (const NDArray&, const octave_uint64&);
#endif
