// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_m_cdm_h)
#define octave_mx_m_cdm_h 1
#include "octave-config.h"
#include "CMatrix.h"
#include "dMatrix.h"
#include "CDiagMatrix.h"
  extern OCTAVE_API ComplexMatrix operator + (const Matrix&, const ComplexDiagMatrix&);
  extern OCTAVE_API ComplexMatrix operator - (const Matrix&, const ComplexDiagMatrix&);
  extern OCTAVE_API ComplexMatrix operator * (const Matrix&, const ComplexDiagMatrix&);
#endif
