// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_i16nda_fnda_h)
#define octave_mx_i16nda_fnda_h 1
#include "octave-config.h"
#include "int16NDArray.h"
#include "fNDArray.h"
  extern OCTAVE_API int16NDArray operator + (const int16NDArray&, const FloatNDArray&);
  extern OCTAVE_API int16NDArray operator - (const int16NDArray&, const FloatNDArray&);
  extern OCTAVE_API int16NDArray product (const int16NDArray&, const FloatNDArray&);
  extern OCTAVE_API int16NDArray quotient (const int16NDArray&, const FloatNDArray&);
  extern OCTAVE_API boolNDArray mx_el_lt (const int16NDArray&, const FloatNDArray&);
  extern OCTAVE_API boolNDArray mx_el_le (const int16NDArray&, const FloatNDArray&);
  extern OCTAVE_API boolNDArray mx_el_ge (const int16NDArray&, const FloatNDArray&);
  extern OCTAVE_API boolNDArray mx_el_gt (const int16NDArray&, const FloatNDArray&);
  extern OCTAVE_API boolNDArray mx_el_eq (const int16NDArray&, const FloatNDArray&);
  extern OCTAVE_API boolNDArray mx_el_ne (const int16NDArray&, const FloatNDArray&);
  extern OCTAVE_API boolNDArray mx_el_and (const int16NDArray&, const FloatNDArray&);
  extern OCTAVE_API boolNDArray mx_el_or (const int16NDArray&, const FloatNDArray&);
  extern OCTAVE_API boolNDArray mx_el_not_and (const int16NDArray&, const FloatNDArray&);
  extern OCTAVE_API boolNDArray mx_el_not_or (const int16NDArray&, const FloatNDArray&);
  extern OCTAVE_API boolNDArray mx_el_and_not (const int16NDArray&, const FloatNDArray&);
  extern OCTAVE_API boolNDArray mx_el_or_not (const int16NDArray&, const FloatNDArray&);
#endif
