// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_fm_pm_h)
#define octave_mx_fm_pm_h 1
#include "octave-config.h"
#include "fMatrix.h"
#include "PermMatrix.h"
  extern OCTAVE_API FloatMatrix operator * (const FloatMatrix&, const PermMatrix&);
#endif
