// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_dm_cm_h)
#define octave_mx_dm_cm_h 1
#include "octave-config.h"
#include "CMatrix.h"
#include "dDiagMatrix.h"
  extern OCTAVE_API ComplexMatrix operator + (const DiagMatrix&, const ComplexMatrix&);
  extern OCTAVE_API ComplexMatrix operator - (const DiagMatrix&, const ComplexMatrix&);
  extern OCTAVE_API ComplexMatrix operator * (const DiagMatrix&, const ComplexMatrix&);
#endif
