// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_fs_i16nda_h)
#define octave_mx_fs_i16nda_h 1
#include "octave-config.h"
#include "int16NDArray.h"
  extern OCTAVE_API int16NDArray operator + (const float&, const int16NDArray&);
  extern OCTAVE_API int16NDArray operator - (const float&, const int16NDArray&);
  extern OCTAVE_API int16NDArray operator * (const float&, const int16NDArray&);
  extern OCTAVE_API int16NDArray operator / (const float&, const int16NDArray&);
  extern OCTAVE_API boolNDArray mx_el_lt (const float&, const int16NDArray&);
  extern OCTAVE_API boolNDArray mx_el_le (const float&, const int16NDArray&);
  extern OCTAVE_API boolNDArray mx_el_ge (const float&, const int16NDArray&);
  extern OCTAVE_API boolNDArray mx_el_gt (const float&, const int16NDArray&);
  extern OCTAVE_API boolNDArray mx_el_eq (const float&, const int16NDArray&);
  extern OCTAVE_API boolNDArray mx_el_ne (const float&, const int16NDArray&);
  extern OCTAVE_API boolNDArray mx_el_and (const float&, const int16NDArray&);
  extern OCTAVE_API boolNDArray mx_el_or (const float&, const int16NDArray&);
  extern OCTAVE_API boolNDArray mx_el_and_not (const float&, const int16NDArray&);
  extern OCTAVE_API boolNDArray mx_el_or_not (const float&, const int16NDArray&);
#endif
