// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_cnda_s_h)
#define octave_mx_cnda_s_h 1
#include "octave-config.h"
#include "CNDArray.h"
  extern OCTAVE_API ComplexNDArray operator + (const ComplexNDArray&, const double&);
  extern OCTAVE_API ComplexNDArray operator - (const ComplexNDArray&, const double&);
  extern OCTAVE_API ComplexNDArray operator * (const ComplexNDArray&, const double&);
  extern OCTAVE_API ComplexNDArray operator / (const ComplexNDArray&, const double&);
  extern OCTAVE_API boolNDArray mx_el_lt (const ComplexNDArray&, const double&);
  extern OCTAVE_API boolNDArray mx_el_le (const ComplexNDArray&, const double&);
  extern OCTAVE_API boolNDArray mx_el_ge (const ComplexNDArray&, const double&);
  extern OCTAVE_API boolNDArray mx_el_gt (const ComplexNDArray&, const double&);
  extern OCTAVE_API boolNDArray mx_el_eq (const ComplexNDArray&, const double&);
  extern OCTAVE_API boolNDArray mx_el_ne (const ComplexNDArray&, const double&);
  extern OCTAVE_API boolNDArray mx_el_and (const ComplexNDArray&, const double&);
  extern OCTAVE_API boolNDArray mx_el_or (const ComplexNDArray&, const double&);
  extern OCTAVE_API boolNDArray mx_el_not_and (const ComplexNDArray&, const double&);
  extern OCTAVE_API boolNDArray mx_el_not_or (const ComplexNDArray&, const double&);
#endif
