// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_nda_ui16_h)
#define octave_mx_nda_ui16_h 1
#include "octave-config.h"
#include "uint16NDArray.h"
#include "dNDArray.h"
#include "oct-inttypes.h"
  extern OCTAVE_API uint16NDArray operator + (const NDArray&, const octave_uint16&);
  extern OCTAVE_API uint16NDArray operator - (const NDArray&, const octave_uint16&);
  extern OCTAVE_API uint16NDArray operator * (const NDArray&, const octave_uint16&);
  extern OCTAVE_API uint16NDArray operator / (const NDArray&, const octave_uint16&);
  extern OCTAVE_API boolNDArray mx_el_lt (const NDArray&, const octave_uint16&);
  extern OCTAVE_API boolNDArray mx_el_le (const NDArray&, const octave_uint16&);
  extern OCTAVE_API boolNDArray mx_el_ge (const NDArray&, const octave_uint16&);
  extern OCTAVE_API boolNDArray mx_el_gt (const NDArray&, const octave_uint16&);
  extern OCTAVE_API boolNDArray mx_el_eq (const NDArray&, const octave_uint16&);
  extern OCTAVE_API boolNDArray mx_el_ne (const NDArray&, const octave_uint16&);
  extern OCTAVE_API boolNDArray mx_el_and (const NDArray&, const octave_uint16&);
  extern OCTAVE_API boolNDArray mx_el_or (const NDArray&, const octave_uint16&);
  extern OCTAVE_API boolNDArray mx_el_not_and (const NDArray&, const octave_uint16&);
  extern OCTAVE_API boolNDArray mx_el_not_or (const NDArray&, const octave_uint16&);
#endif
