// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_fm_fcs_h)
#define octave_mx_fm_fcs_h 1
#include "octave-config.h"
#include "fCMatrix.h"
#include "fMatrix.h"
#include "oct-cmplx.h"
  extern OCTAVE_API FloatComplexMatrix operator + (const FloatMatrix&, const FloatComplex&);
  extern OCTAVE_API FloatComplexMatrix operator - (const FloatMatrix&, const FloatComplex&);
  extern OCTAVE_API FloatComplexMatrix operator * (const FloatMatrix&, const FloatComplex&);
  extern OCTAVE_API FloatComplexMatrix operator / (const FloatMatrix&, const FloatComplex&);
  extern OCTAVE_API boolMatrix mx_el_lt (const FloatMatrix&, const FloatComplex&);
  extern OCTAVE_API boolMatrix mx_el_le (const FloatMatrix&, const FloatComplex&);
  extern OCTAVE_API boolMatrix mx_el_ge (const FloatMatrix&, const FloatComplex&);
  extern OCTAVE_API boolMatrix mx_el_gt (const FloatMatrix&, const FloatComplex&);
  extern OCTAVE_API boolMatrix mx_el_eq (const FloatMatrix&, const FloatComplex&);
  extern OCTAVE_API boolMatrix mx_el_ne (const FloatMatrix&, const FloatComplex&);
  extern OCTAVE_API boolMatrix mx_el_and (const FloatMatrix&, const FloatComplex&);
  extern OCTAVE_API boolMatrix mx_el_or (const FloatMatrix&, const FloatComplex&);
#endif
