// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_vx_ccv_s_h)
#define octave_vx_ccv_s_h 1
#include "octave-config.h"
#include "CColVector.h"
  extern OCTAVE_API ComplexColumnVector operator + (const ComplexColumnVector&, const double&);
  extern OCTAVE_API ComplexColumnVector operator - (const ComplexColumnVector&, const double&);
  extern OCTAVE_API ComplexColumnVector operator * (const ComplexColumnVector&, const double&);
  extern OCTAVE_API ComplexColumnVector operator / (const ComplexColumnVector&, const double&);
#endif
