// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_smx_m_sm_h)
#define octave_smx_m_sm_h 1
#include "octave-config.h"
#include "dMatrix.h"
#include "dSparse.h"
#include "Sparse-op-defs.h"
  extern OCTAVE_API Matrix operator + (const Matrix&, const SparseMatrix&);
  extern OCTAVE_API Matrix operator - (const Matrix&, const SparseMatrix&);
  extern OCTAVE_API SparseMatrix product (const Matrix&, const SparseMatrix&);
  extern OCTAVE_API SparseMatrix quotient (const Matrix&, const SparseMatrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_lt (const Matrix&, const SparseMatrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_le (const Matrix&, const SparseMatrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_ge (const Matrix&, const SparseMatrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_gt (const Matrix&, const SparseMatrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_eq (const Matrix&, const SparseMatrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_ne (const Matrix&, const SparseMatrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_and (const Matrix&, const SparseMatrix&);
  extern OCTAVE_API SparseBoolMatrix mx_el_or (const Matrix&, const SparseMatrix&);
#endif
