// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_vx_fcrv_frv_h)
#define octave_vx_fcrv_frv_h 1
#include "octave-config.h"
#include "fCRowVector.h"
#include "fRowVector.h"
  extern OCTAVE_API FloatComplexRowVector operator + (const FloatComplexRowVector&, const FloatRowVector&);
  extern OCTAVE_API FloatComplexRowVector operator - (const FloatComplexRowVector&, const FloatRowVector&);
  extern OCTAVE_API FloatComplexRowVector product (const FloatComplexRowVector&, const FloatRowVector&);
  extern OCTAVE_API FloatComplexRowVector quotient (const FloatComplexRowVector&, const FloatRowVector&);
#endif
