// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_vx_rv_crv_h)
#define octave_vx_rv_crv_h 1
#include "octave-config.h"
#include "CRowVector.h"
#include "dRowVector.h"
  extern OCTAVE_API ComplexRowVector operator + (const RowVector&, const ComplexRowVector&);
  extern OCTAVE_API ComplexRowVector operator - (const RowVector&, const ComplexRowVector&);
  extern OCTAVE_API ComplexRowVector product (const RowVector&, const ComplexRowVector&);
  extern OCTAVE_API ComplexRowVector quotient (const RowVector&, const ComplexRowVector&);
#endif
