// DO NOT EDIT -- generated by mk-ops.awk
#if ! defined (octave_mx_fcnda_fs_h)
#define octave_mx_fcnda_fs_h 1
#include "octave-config.h"
#include "fCNDArray.h"
  extern OCTAVE_API FloatComplexNDArray operator + (const FloatComplexNDArray&, const float&);
  extern OCTAVE_API FloatComplexNDArray operator - (const FloatComplexNDArray&, const float&);
  extern OCTAVE_API FloatComplexNDArray operator * (const FloatComplexNDArray&, const float&);
  extern OCTAVE_API FloatComplexNDArray operator / (const FloatComplexNDArray&, const float&);
  extern OCTAVE_API boolNDArray mx_el_lt (const FloatComplexNDArray&, const float&);
  extern OCTAVE_API boolNDArray mx_el_le (const FloatComplexNDArray&, const float&);
  extern OCTAVE_API boolNDArray mx_el_ge (const FloatComplexNDArray&, const float&);
  extern OCTAVE_API boolNDArray mx_el_gt (const FloatComplexNDArray&, const float&);
  extern OCTAVE_API boolNDArray mx_el_eq (const FloatComplexNDArray&, const float&);
  extern OCTAVE_API boolNDArray mx_el_ne (const FloatComplexNDArray&, const float&);
  extern OCTAVE_API boolNDArray mx_el_and (const FloatComplexNDArray&, const float&);
  extern OCTAVE_API boolNDArray mx_el_or (const FloatComplexNDArray&, const float&);
  extern OCTAVE_API boolNDArray mx_el_not_and (const FloatComplexNDArray&, const float&);
  extern OCTAVE_API boolNDArray mx_el_not_or (const FloatComplexNDArray&, const float&);
#endif
